@@ -996,7 +996,7 @@ class ClientGroup(object):
                     }]
                 },
                 "clientGroup": {
-                    "newName": self.clientgroup_name
+                    "newName": self.name
                 }
             }
         }
@@ -1016,7 +1016,7 @@ class ClientGroup(object):
                     }]
                 },
                 "clientGroup": {
-                    "newName": self.clientgroup_name
+                    "newName": self.name
                 }
             }
         }
@@ -1180,7 +1180,7 @@ class ClientGroup(object):
                 raise SDKException('ClientGroup', '102', 'No valid clients were found')
 
             output = self._update(
-                clientgroup_name=self.clientgroup_name,
+                clientgroup_name=self.name,
                 clientgroup_description=self.description,
                 associated_clients=validated_clients_list,
                 operation_type=operation_type
@@ -1591,7 +1591,7 @@ class ClientGroup(object):
         """Sets the description of the clientgroup as the value provided in input."""
         if isinstance(value, basestring):
             output = self._update(
-                clientgroup_name=self.clientgroup_name,
+                clientgroup_name=self.name,
                 clientgroup_description=value,
                 associated_clients=self._associated_clients
             )
@@ -1657,7 +1657,7 @@ class ClientGroup(object):
                     if failed to remove all clients from client group
         """
         output = self._update(
-            clientgroup_name=self.clientgroup_name,
+            clientgroup_name=self.name,
             clientgroup_description=self.description,
             associated_clients=self._associated_clients,
             operation_type="CLEAR"
@@ -1752,7 +1752,7 @@ class ClientGroup(object):
         """
         install = Install(self._commcell_object)
         return install.push_servicepack_and_hotfix(
-            client_computer_groups=[self.clientgroup_name],
+            client_computer_groups=[self.name],
             reboot_client=reboot_client,
             run_db_maintenance=run_db_maintenance)
 
@@ -1781,7 +1781,7 @@ class ClientGroup(object):
             "clientGroupOperationType": 2,
             "clientGroupDetail": {
                 "clientGroup": {
-                    "clientGroupName": self.clientgroup_name
+                    "clientGroupName": self.name
                 }
             }
         }
