@@ -51,6 +51,7 @@ SERVICES_DICT_TEMPLATE = {
     'GET_ALL_CLIENTS': '{0}Client',
     'GET_VIRTUAL_CLIENTS': '{0}Client?PseudoClientType=VSPseudo',
     'CLIENT': '{0}Client/%s',
+    'FILTER_CLIENTS':'{0}Client?%s',
     'GET_ALL_CLIENTS_PLUS_HIDDEN': '{0}Client?hiddenclients=true',
     'GET_ALL_PSEUDO_CLIENTS': '{0}Client?PseudoClientType',
     'CHECK_READINESS': '{0}Client/%s/CheckReadiness?network=%s&resourceCapacity=%s'
@@ -82,6 +83,9 @@ SERVICES_DICT_TEMPLATE = {
     'ALL_JOBS': '{0}Jobs',
     'JOB_MANAGEMENT_SETTINGS': '{0}CommServ/JobManagementSetting',
 
+    'ENABLE_SHARED_LAPTOP': '{0}Commcell/Properties/SharedLaptopUsage/Action/Enable',
+    'DISABLE_SHARED_LAPTOP': '{0}Commcell/Properties/SharedLaptopUsage/Action/Disable',
+
     'GET_MEDIA_AGENTS': '{0}V2/MediaAgents',
     'LIBRARY': '{0}Library',
     'GET_LIBRARY_PROPERTIES': '{0}Library/%s',
@@ -94,6 +98,7 @@ SERVICES_DICT_TEMPLATE = {
     'CREATE_UPDATE_SCHEDULE_POLICY': '{0}Task',
     'GET_SCHEDULE_POLICY': '{0}SchedulePolicy/%s',
     'MEDIA_AGENT': '{0}MediaAgent/%s',
+    'CLOUD_MEDIA_AGENT': '{0}MediaAgent/%s/CloudVMPowerManagement',
     'STORAGE_POLICY_COPY': '{0}V2/StoragePolicy/%s/Copy/%s',
 
     'GET_ALL_ALERTS': '{0}AlertRule',
@@ -102,6 +107,8 @@ SERVICES_DICT_TEMPLATE = {
     'DELETE_BLR_PAIR': '{0}Replications/Monitors/continuous/%s',
     'GRANULAR_BLR_POINTS': '{0}/Replications/Monitors/continuous/VmScale?destProxyClientId=%s&subclientId=%s&vmUuid=%s',
 
+    'GET_VM_BROWSE': '{0}/VMBrowse?inventoryPath=%%5CFOLDER%%3AApplications%%3AApplications&PseudoClientId=%s',
+
     'MODIFY_ALERT': '{0}AlertRule/%s/Action/Modify',
     'GET_ALL_CONSOLE_ALERTS': '{0}Alert?pageNo=%s&pageCount=%s',
     'ENABLE_ALERT_NOTIFICATION': '{0}AlertRule/%s/notificationType/%s/Action/Enable',
@@ -135,6 +142,8 @@ SERVICES_DICT_TEMPLATE = {
 
     'BROWSE': '{0}DoBrowse',
     'RESTORE': '{0}CreateTask',
+    'SQL_CLONES': '{0}sql/clones',
+    'SQL_DATABASES': '{0}sql/databases?databaseName=%s',
 
     'GET_WORKFLOWS': '{0}Workflow',
     'DEPLOY_WORKFLOW': '{0}Workflow/%s/action/deploy',
@@ -185,6 +194,9 @@ SERVICES_DICT_TEMPLATE = {
     'DELETE_HANDLER': '{0}dcube/deletehandler/%s',
     'SHARE_HANDLER': '{0}dcube/share/handler',
     'SHARE_DATASOURCE': '{0}dcube/share/datasource',
+    'GET_CONTENT_ANALYZER_CLOUD':'{0}getContentAnalyzerCloud',
+    'ACTIVATE_ENTITIES':'{0}dcube/entity',
+    'ACTIVATE_ENTITY':'{0}dcube/entity/%s',
 
     'GLOBAL_FILTER': '{0}GlobalFilter',
     'RESTORE_OPTIONS': '{0}Restore/GetDestinationsToRestore?clientId=0&appId=%s&flag=8',
@@ -198,11 +210,17 @@ SERVICES_DICT_TEMPLATE = {
     'ADD_USERS_TO_PLAN': '{0}V2/Plan/%s/Users',
     'GET_PLAN_TEMPLATE': '{0}V2/Plan/template?type=%s&subType=%s',
     'ELIGIBLE_PLANS': '{0}V2/Plan/Eligible?%s',
+    'ASSOCIATED_ENTITIES': '{0}V2/Plan/%s/AssociatedEntities',
 
     'DOMAIN_CONTROLER': '{0}CommCell/DomainController',
     'DELETE_DOMAIN_CONTROLER': '{0}CommCell/DomainController/%s',
+
     'DRBACKUP': '{0}/CommServ/DRBackup',
     'DISASTER_RECOVERY_PROPERTIES': '{0}/Commcell/DRBackup/Properties',
+    'CVDRBACKUP_STATUS': '{0}/cvdrbackup/status?commcellid=%s',
+    'CVDRBACKUP_INFO': '{0}/cvdrbackup/info',
+    'CVDRBACKUP_DOWNLOAD': '{0}/cvdrbackup/download',
+
     'ORACLE_INSTANCE_BROWSE': '{0}Instance/DBBrowse/%s',
 
     'METRICS': '{0}CommServ/MetricsReporting',
@@ -252,6 +270,8 @@ SERVICES_DICT_TEMPLATE = {
     'STORAGE_POOL': '{0}StoragePool',
     'GET_STORAGE_POOL': '{0}StoragePool/%s',
     'ADD_STORAGE_POOL': '{0}StoragePool?Action=create',
+    'DELETE_STORAGE_POOL': '{0}StoragePool/%s',
+    'EDIT_STORAGE_POOL': '{0}StoragePool?Action=edit',
 
     'GET_ALL_MONITORING_POLICIES': '{0}logmonitoring/monitoringpolicy',
     'GET_ALL_ANALYTICS_SERVERS': '{0}AnalyticsServers',
@@ -270,6 +290,12 @@ SERVICES_DICT_TEMPLATE = {
     'REPLICATION_MONITOR': '{0}Replications/Monitors/streaming?subclientId=0',
     'RPSTORE': '{0}Replications/RPStore',
 
+    'CREATE_PSEUDO_CLIENT':'{0}pseudoClient',
+    'GET_OFFICE_365_ENTITIES':'{0}Office365/entities',
+    'CLOUD_DISCOVERY':'{0}Instance/%s/CloudDiscovery?clientId=%s&appType=%s',
+    'USER_POLICY_ASSOCIATION':'{0}Office365/CloudApps/UserPolicyAssociation',
+    'UPDATE_USER_POLICY_ASSOCIATION':'{0}Office365/CloudApps/UpdateUserPolicyAssociation',
+
     'ADD_EXCHANGE': '{0}pseudoClient',
     'CREATE_CONFIGURATION_POLICIES': '{0}ConfigurationPolicies',
     'GET_CONFIGURATION_POLICIES': '{0}ConfigurationPolicies?policyType=email',
@@ -297,6 +323,7 @@ SERVICES_DICT_TEMPLATE = {
     'IDENTITY_APPS': '{0}ThirdParty/App',
 
     'GLOBAL_PARAM': '{0}/setGlobalParam',
+    'GET_GLOBAL_PARAM': '{0}/CommServ/AddRemoveSoftware/CommServeSoftwareCache',
 
     'SNAP_OPERATIONS': '{0}/Snaps/Operations',
     'STORAGE_ARRAYS': '{0}/StorageArrays',
@@ -333,7 +360,15 @@ SERVICES_DICT_TEMPLATE = {
         '{0}/VMAllocationPolicy?showResourceGroupPolicy=true&showNonResourceGroupPolicy=false&deep=true',
     'GET_RECOVERY_TARGET': '{0}/VMAllocationPolicy/%s',
 
-    'RETIRE': '{0}Client/%s/Retire'
+    'RETIRE': '{0}Client/%s/Retire',
+
+    'CLOUD_CREATE': '{0}cloud/create',
+    'CLOUD_MODIFY': '{0}cloud/modify',
+    'CLOUD_DELETE': '{0}cloud/delete',
+    'CLOUD_ROLE_UPDATE': '{0}cloud/role/update',
+    'CLOUD_NODE_UPDATE': '{0}cloud/node/update',
+    'GET_ALL_INDEX_SERVERS': '{0}dcube/getAnalyticsEngine?retrieveall=true',
+    'GET_ALL_ROLES': '{0}IndexingGateway/GetAnalyticsRolesInfo',
 }
 
 
