@@ -42,6 +42,9 @@ FileSystemSubclient:
 
     run_backup_copy()                   --  Runs the backup copy job from Subclient
 
+    restore_in_place()                  --  Restores the files/folders
+                                            specified in the input paths list to the same location.
+
     restore_out_of_place()              --  Restores the files/folders specified in the input paths list
                                             to the input client, at the specified destionation location
 
@@ -120,6 +123,10 @@ FileSystemSubclient Instance Attributes:
 
     **backup_savf_file_data**             --  Sets the savf file data property for ibmi backup.
 
+	**pre_post_commands**				  --  Sets the pre/post commands for the subclient.
+
+	**backup_nodes**                      --  Sets backup nodes for FS Agent under Network Share clients.
+
 """
 
 from __future__ import unicode_literals
@@ -291,23 +298,23 @@ class FileSystemSubclient(Subclient):
         final_dict = super(FileSystemSubclient, self)._common_backup_options(options)
 
         common_options = {
-                "jobDescription": options.get('job_description', ""),
-                "jobRetryOpts": {
-                    "killRunningJobWhenTotalRunningTimeExpires": options.get(
-                        'kill_running_job_when_total_running_time_expires', False),
-                    "numberOfRetries": options.get('number_of_retries', 0),
-                    "enableNumberOfRetries": options.get('enable_number_of_retries', False),
-                    "runningTime": {
-                        "enableTotalRunningTime": options.get('enable_total_running_time', False),
-                        "totalRunningTime": options.get('total_running_time', 3600)
-                    }
-                },
-                "startUpOpts": {
-                    "startInSuspendedState": options.get('start_in_suspended_state', False),
-                    "useDefaultPriority": options.get('use_default_priority', True),
-                    "priority": options.get('priority', 166)
+            "jobDescription": options.get('job_description', ""),
+            "jobRetryOpts": {
+                "killRunningJobWhenTotalRunningTimeExpires": options.get(
+                    'kill_running_job_when_total_running_time_expires', False),
+                "numberOfRetries": options.get('number_of_retries', 0),
+                "enableNumberOfRetries": options.get('enable_number_of_retries', False),
+                "runningTime": {
+                    "enableTotalRunningTime": options.get('enable_total_running_time', False),
+                    "totalRunningTime": options.get('total_running_time', 3600)
                 }
+            },
+            "startUpOpts": {
+                "startInSuspendedState": options.get('start_in_suspended_state', False),
+                "useDefaultPriority": options.get('use_default_priority', True),
+                "priority": options.get('priority', 166)
             }
+        }
 
         return common_options
 
@@ -1554,6 +1561,94 @@ class FileSystemSubclient(Subclient):
 
         return self._process_backup_response(flag, response)
 
+    def restore_in_place(
+            self,
+            paths,
+            overwrite=True,
+            restore_data_and_acl=True,
+            copy_precedence=None,
+            from_time=None,
+            to_time=None,
+            fs_options=None,
+            schedule_pattern=None,
+            proxy_client=None):
+        """Restores the files/folders specified in the input paths list to the same location.
+
+            Args:
+                paths                   (list)  --  list of full paths of files/folders to restore
+
+                overwrite               (bool)  --  unconditional overwrite files during restore
+                    default: True
+
+                restore_data_and_acl    (bool)  --  restore data and ACL files
+                    default: True
+
+                copy_precedence         (int)   --  copy precedence value of storage policy copy
+                    default: None
+
+                from_time           (str)       --  time to retore the contents after
+                        format: YYYY-MM-DD HH:MM:SS
+
+                    default: None
+
+                to_time           (str)         --  time to retore the contents before
+                        format: YYYY-MM-DD HH:MM:SS
+
+                    default: None
+
+                fs_options      (dict)          -- dictionary that includes all advanced options
+                    options:
+                        all_versions        : if set to True restores all the versions of the
+                                                specified file
+                        versions            : list of version numbers to be backed up
+                        validate_only       : To validate data backed up for restore
+
+
+                schedule_pattern (dict) -- scheduling options to be included for the task
+
+                        Please refer schedules.schedulePattern.createSchedule()
+                                                                    doc for the types of Jsons
+
+                schedule_pattern (dict) -- scheduling options to be included for the task
+
+                        Please refer schedules.schedulePattern.createSchedule()
+                                                                    doc for the types of Jsons
+
+                proxy_client    (str)          -- Proxy client used during FS under NAS operations
+
+            Returns:
+                object - instance of the Job class for this restore job if its an immediate Job
+                         instance of the Schedule class for this restore job if its a scheduled Job
+
+            Raises:
+                SDKException:
+                    if paths is not a list
+
+                    if failed to initialize job
+
+                    if response is empty
+
+                    if response is not success
+        """
+        self._backupset_object._instance_object._restore_association = self._subClientEntity
+
+        if fs_options is not None and fs_options.get('no_of_streams', 1) > 1 and not fs_options.get('destination_appTypeId', False):
+            fs_options['destination_appTypeId'] = int(self._client_object.agents.all_agents.get('file system', self._client_object.agents.all_agents.get('windows file system', self._client_object.agents.all_agents.get('linux file system', self._client_object.agents.all_agents.get('big data apps', self._client_object.agents.all_agents.get('cloud apps'))))))
+            if not fs_options['destination_appTypeId']:
+                del fs_options['destination_appTypeId']
+
+        return super(FileSystemSubclient, self).restore_in_place(
+            paths=paths,
+            overwrite=overwrite,
+            restore_data_and_acl=restore_data_and_acl,
+            copy_precedence=copy_precedence,
+            from_time=from_time,
+            to_time=to_time,
+            fs_options=fs_options,
+            schedule_pattern=schedule_pattern,
+            proxy_client=proxy_client
+        )
+
     def restore_out_of_place(
             self,
             client,
@@ -1565,7 +1660,8 @@ class FileSystemSubclient(Subclient):
             from_time=None,
             to_time=None,
             fs_options=None,
-            schedule_pattern=None):
+            schedule_pattern=None
+    ):
         """Restores the files/folders specified in the input paths list to the input client,
             at the specified destionation location.
 
@@ -1598,18 +1694,29 @@ class FileSystemSubclient(Subclient):
                     default: None
 
                 fs_options      (dict)          -- dictionary that includes all advanced options
+
                     options:
+
                         preserve_level          : preserve level option to set in restore
+
                         proxy_client            : proxy that needed to be used for restore
+
                         impersonate_user        : Impersonate user options for restore
+
                         impersonate_password    : Impersonate password option for restore
                         in base64 encoded form
+
                         all_versions            : if set to True restores all the versions of the
                         specified file
+
                         versions                : list of version numbers to be backed up
+
                         media_agent             : Media Agent need to be used for Browse and restore
+
                         is_vlr_restore          : sets if the restore job is to be triggered as vlr
+
                         validate_only           : To validate data backed up for restore
+
                         instant_clone_options   : Options for FS clone found on Command Center, the value must be
                         a dictionary containing the following key value pairs.
 
@@ -1628,6 +1735,8 @@ class FileSystemSubclient(Subclient):
                             clone_cleanup_script    (str)   --  The script that will run after clean up.
                             This is an OPTIONAL key.
 
+                        no_of_streams   (int)       -- Number of streams to be used for restore
+
                 schedule_pattern (dict) -- scheduling options to be included for the task
 
                         Please refer schedules.schedulePattern.createSchedule()
@@ -1653,7 +1762,18 @@ class FileSystemSubclient(Subclient):
         """
         self._backupset_object._instance_object._restore_association = self._subClientEntity
 
-        # check to find whether file level Restore/ Volume level restore for blocklvel.
+        if not isinstance(client, (basestring, Client)):
+            raise SDKException('Subclient', '101')
+
+        if isinstance(client, basestring):
+            client = Client(self._commcell_object, client)
+
+        if fs_options is not None and fs_options.get('no_of_streams', 1) > 1 and not fs_options.get('destination_appTypeId', False):
+            fs_options['destination_appTypeId'] = int(client.agents.all_agents.get('file system', client.agents.all_agents.get('windows file system', client.agents.all_agents.get('linux file system', client.agents.all_agents.get('big data apps', client.agents.all_agents.get('cloud apps'))))))
+            if not fs_options['destination_appTypeId']:
+                del fs_options['destination_appTypeId']
+
+            # check to find whether file level Restore/ Volume level restore for blocklevel.
 
         if fs_options is not None and fs_options.get('is_vlr_restore', False):
             if not (isinstance(paths, list) and
@@ -1675,7 +1795,8 @@ class FileSystemSubclient(Subclient):
                 from_time=from_time,
                 to_time=to_time,
                 destPath=destination_path,
-                restore_option=fs_options)
+                restore_option=fs_options
+            )
 
             request_json['taskInfo']['subTasks'][0]['options']['restoreOptions'].update(
                 self._vlr_restore_options_dict)
@@ -1696,7 +1817,8 @@ class FileSystemSubclient(Subclient):
                 from_time=from_time,
                 to_time=to_time,
                 fs_options=fs_options,
-                schedule_pattern=schedule_pattern)
+                schedule_pattern=schedule_pattern
+            )
 
     @property
     def catalog_acl(self):
@@ -1906,3 +2028,81 @@ class FileSystemSubclient(Subclient):
                 None
         """
         self._set_subclient_properties("_fsSubClientProp['backupSaveFileData']", value)
+
+    @property
+    def pre_post_commands(self):
+        """
+         Return the prep_post commands set for a subclient
+
+        Returns:
+            (dict)  --  All the pre/post commands
+        """
+        pre_scan_command = self._commonProperties["prepostProcess"]["preScanCommand"]
+        post_scan_command = self._commonProperties["prepostProcess"]["postScanCommand"]
+        pre_backup_command = self._commonProperties["prepostProcess"]["preBackupCommand"]
+        post_backup_command = self._commonProperties["prepostProcess"]["postBackupCommand"]
+
+        pre_post_commands = {'pre_scan_command': pre_scan_command, 'post_scan_command': post_scan_command, 'pre_backup_command': pre_backup_command, 'post_backup_command': post_backup_command}
+
+        return pre_post_commands
+
+    @pre_post_commands.setter
+    def pre_post_commands(self, value):
+        """
+        Sets the pre post commands on a subclient
+        Args:
+
+            value   (dict)      --  Specifies the pre.post commands to be set
+
+                pre_scan_command    (str)       --     The pre scan command to be set
+
+                post_scan_command   (str)       --     The post scan command to be set
+
+                pre_backup_command  (str)       --     The pre backup command to be set
+
+                post_backup_command (str)       --     The post backup command to be set
+
+            Returns:
+                None
+
+            Raises:
+                None
+        """
+        if isinstance(value, dict):
+
+            pre_post_process = self._commonProperties["prepostProcess"]
+
+            pre_post_process["preScanCommand"] = value.get("pre_scan_command", "")
+            pre_post_process["postScanCommand"] = value.get("post_scan_command", "")
+            pre_post_process["preBackupCommand"] = value.get("pre_backup_command", "")
+            pre_post_process["postBackupCommand"] = value.get("post_backup_command", "")
+
+            self._set_subclient_properties('_commonProperties["prepostProcess"]', pre_post_process)
+
+        else:
+            raise SDKException('Subclient', '101')
+
+    @property
+    def backup_nodes(self):
+        """
+        Gets the backup nodes for FS Agent under Network Share Clients.
+        """
+        return self._fsSubClientProp["backupConfiguration"]["backupDataAccessNodes"]
+
+    @backup_nodes.setter
+    def backup_nodes(self, value):
+        """
+        Sets the backup nodes for FS Agent under Network Share Clients.
+
+            Args:
+                value  (list)   --  Specifies the nodes, a list of strings, values are data access node host names.
+        """
+
+        update_properties = self.properties
+
+        access_nodes = []
+        for access_node in value:
+            access_nodes.append({"clientName": access_node})
+
+        update_properties["fsSubClientProp"]["backupConfiguration"] = {"backupDataAccessNodes": access_nodes}
+        self.update_properties(update_properties)
